traditional_chinese_word_dict = {
    # Main
    "application_name": "JEditor",
    # Tab
    "tab_name_editor": "編輯器",
    "tab_name_web_browser": "網頁瀏覽器",
    "tab_name_frontengine": "前景引擎",
    "tab_name_re_edge_gpt": "Re-EdgeGPT",
    # Browser
    "browser_download_detail": "下載資訊",
    "browser_find_text": "搜尋文字",
    "browser_find_text_input": "輸入要尋找的文字",
    "browser_back_button": "上一頁",
    "browser_forward_button": "下一頁",
    "browser_reload_button": "重新載入",
    "browser_search_button": "搜尋",
    # Dialog
    "dialog_create_file": "建立檔案",
    "dialog_input_file_name": "請輸入正確格式的檔案名稱",
    "dialog_search_next": "搜尋下一個",
    "dialog_search_back": "搜尋上一個",
    # Editor
    "editor_code_result": "程式運行結果",
    "editor_format_check": "程式格式檢查",
    # Check Code Style Menu
    "check_code_style_menu_label": "程式碼格式檢查器",
    "yapf_reformat_label": "yapf",
    "black_reformat_label": "Black",
    "reformat_json_label": "重新格式化 JSON",
    # Dock Menu
    "dock_menu_label": "區域",
    "dock_browser_label": "新瀏覽器區域",
    "dock_stackoverflow_label": "新 Stackoverflow 區域",
    "dock_editor_label": "新編輯器區域",
    "dock_frontengine_label": "新前景引擎區域",
    "dock_ipython_label": "New Ipython(Jupyter)",
    "dock_editor_title": "編輯器",
    "dock_frontengine_title": "前景引擎",
    "dock_browser_title": "瀏覽器",
    "dock_ipython_title": "IPython(Jupyter)",
    # File Menu
    "file_menu_label": "檔案",
    "file_menu_new_file_label": "新檔案",
    "file_menu_open_file_label": "開啟檔案",
    "file_menu_open_folder_label": "開啟資料夾",
    "file_menu_save_file_label": "儲存檔案",
    "file_menu_encoding_label": "編碼",
    "file_menu_font_label": "字體",
    "file_menu_font_size_label": "字體大小",
    # Help Menu
    "help_menu_label": "幫助",
    "help_menu_open_github_label": "GitHub",
    "help_menu_open_doc_label": "文件",
    "help_menu_open_about_label": "關於",
    "help_menu_open_re_edge_gpt_doc_label": "開啟 ReEdgeGPT 文件",
    "help_menu_open_re_edge_gpt_github_label": "開啟 ReEdgeGPT GitHub",
    # Python Env Menu
    "python_env_menu_label": "Python 環境",
    "python_env_menu_create_venv_label": "建立 venv 虛擬環境",
    "python_env_menu_pip_upgrade_label": "升級與安裝套件",
    "python_env_menu_pip_label": "安裝套件",
    "python_env_menu_choose_interpreter_label": "選擇 Python 直譯器",
    "python_env_menu_creating_venv_message": "建立虛擬環境中, 請等待 Shell exit code",
    "python_env_menu_venv_exists": "虛擬環境已經存在",
    "python_env_menu_please_create_venv": "請先建立虛擬環境",
    "python_env_menu_install_package_messagebox_title": "安裝套件",
    "python_env_menu_install_package_messagebox_label": "想安裝甚麼套件?",
    "python_env_menu_install_or_update_package_messagebox_label": "想安裝或升級甚麼套件",
    # Run Menu
    "run_menu_label": "運行",
    "run_menu_run_program_label": "運行程式",
    "run_menu_run_debugger": "運行除錯器",
    "run_menu_run_on_shell_label": "在終端運行",
    "run_menu_clear_result_label": "清除結果",
    "run_menu_stop_all_program_label": "關閉所有程式",
    "run_menu_stop_program_label": "關閉當前程式",
    "run_menu_run_help_label": "運行幫助",
    "run_menu_shell_help_label": "終端運行幫助",
    "please_stop_current_running_shell": "請先關閉目前正在運行的終端",
    "please_stop_current_running_program": "請先關閉目前正在運行的程式",
    "run_menu_run_help_tip": """
如果不能運行程式, 請檢查是否有 Python 直譯器,如有 Python 衝突請選擇其他直譯器.
        """,
    "run_menu_shell_run_tip": """
如果有亂碼等問題請重新選擇編碼至與系統 Shell 編碼相同
        """,
    # Debugger
    "editor_debugger_input_title_label": "除錯器",
    "show_debugger_input": "開啟除錯輸入",
    # Shell and menu
    "editor_shell_input_title_label": "Shell",
    "show_shell_input": "開啟 Shell 輸入",
    # Program and menu
    "show_program_input": "開啟程式輸入",
    "process_input_send_command": "傳送指令",
    # Style Menu
    "style_menu_label": "UI 風格",
    # Tab Menu
    "tab_menu_label": "分頁",
    "tab_menu_add_editor_label": "新增編輯器分頁",
    "tab_menu_add_frontengine_label": "新增前景引擎分頁",
    "tab_menu_add_web_label": "新增瀏覽器分頁",
    "tab_menu_add_stackoverflow_label": "新增 Stackoverflow 分頁",
    "tab_menu_editor_tab_name": "編輯器",
    "tab_menu_frontengine_tab_name": "前景引擎",
    "tab_menu_web_tab_name": "瀏覽器",
    "tab_menu_stackoverflow_tab_name": "Stackoverflow",
    "tab_menu_ipython_tab_name": "IPython(Jupyter)",
    "tab_menu_re_re_edge_gpt_tab_name": "ReEdgeGPT",
    # Text Menu
    "text_menu_label": "文字",
    "text_menu_label_font": "字體",
    "text_menu_label_font_size": "字體大小",
    # System tray
    "system_tray_hide": "隱藏",
    "system_tray_maximized": "最大化",
    "system_tray_normal": "正常顯示",
    "system_tray_close": "關閉",
    # Language Menu
    "language_menu_label": "語言",
    "language_menu_bar_english": "英文",
    "language_menu_bar_traditional_chinese": "繁體中文",
    "language_menu_bar_please_restart_messagebox": "請重啟以應用更改",
    # Qtconsole
    "please_install_qtcontsole_label": "請先 python -m pip install qtconsole",
}
